import React, {
  useCallback,
  useEffect,
  useMemo,
  useRef,
  useState,
} from 'react';

import {
  Anchor,
  Button,
  Popover,
  Radio,
  Result,
  Spin,
  Tabs,
  Divider,
  Dropdown,
  Menu
} from 'antd';
import Icon from '@ant-design/icons';
import { store } from 'microser-data';
import classNames from 'classnames';
import { ReactSVG } from 'react-svg'
import Cookies from 'js-cookie';
import { dom } from 'kit_utils';
import { connect } from 'react-redux';
import { useMountedState } from 'react-use';

import RightContent from '@/components/GlobalHeader/ExtendRightContent';
import Panel from '@/components/SliderPanel/MinPanel';
import { actionCreators } from '@/containers/App/store';
import Authorized from '@/utils/Authorized';
import {
  findPathByLeafId,
  getBaseName,
} from '@/utils/utils';
import {
  LeftOutlined,
  LoadingOutlined,
  MenuFoldOutlined,
  DownOutlined
} from '@ant-design/icons';
import ProLayout, { getPageTitle, PageContainer } from '@ant-design/pro-layout';
import { getMatchMenu } from '@umijs/route-utils';
// import { renderRoutes } from 'react-router-config';
import { useHistory } from '@wisdom-utils/runtime';

import pkg from '../../package.json';
import SecurityLayout from './SecurityLayout';
import Site from './Site';
import styles from './UserLayout.less';
import layoutStyles from './BasicLayout.less';

const { TabPane } = Tabs;
const { layout: defaultSetting } = require('../../config/config');
// import Login from '../pages/user/login/login';
const antIcon = <LoadingOutlined style={{ fontSize: 12 }} spin />;

const homeSvg = () => (
  <svg t="1631169883330" class="icon" viewBox="0 0 1024 1024" version="1.1" xmlns="http://www.w3.org/2000/svg" p-id="2203" width="20" height="20"><path d="M423.9 487H221.7c-34.9 0-63.4-28.4-63.4-63.4V221.5c0-34.9 28.4-63.4 63.4-63.4h202.2c34.9 0 63.4 28.4 63.4 63.4v202.2c-0.1 34.9-28.5 63.3-63.4 63.3z m0-63.3v31.7-31.7zM221.7 221.5v202.2h202.1l0.1-202.2H221.7zM803.1 487H600.9c-34.9 0-63.4-28.4-63.4-63.4V221.5c0-34.9 28.4-63.4 63.4-63.4h202.2c34.9 0 63.4 28.4 63.4 63.4v202.2c0 34.9-28.5 63.3-63.4 63.3z m0-63.3v31.7-31.7zM600.9 221.5v202.2H803l0.1-202.2H600.9zM423.9 865.8H221.7c-34.9 0-63.4-28.4-63.4-63.4V600.3c0-34.9 28.4-63.4 63.4-63.4h202.2c34.9 0 63.4 28.4 63.4 63.4v202.2c-0.1 34.9-28.5 63.3-63.4 63.3z m0-63.3v31.7-31.7zM221.7 600.3v202.2h202.1l0.1-202.2H221.7zM736.9 865.8h-69.7c-71.4 0-129.6-58.1-129.6-129.6v-69.7c0-71.4 58.1-129.6 129.6-129.6h69.7c71.4 0 129.6 58.1 129.6 129.6v69.7c0 71.5-58.2 129.6-129.6 129.6z m-69.8-265.5c-36.5 0-66.2 29.7-66.2 66.2v69.7c0 36.5 29.7 66.2 66.2 66.2h69.7c36.5 0 66.2-29.7 66.2-66.2v-69.7c0-36.5-29.7-66.2-66.2-66.2h-69.7z" fill="#B2B2B2" p-id="2204"></path></svg>
);

const arrowSvg = ({fillColor = '#fff'}) => (
  <svg t="1543324489942" 
        class="icon" viewBox="0 0 1024 1024" 
        version="1.1" xmlns="http://www.w3.org/2000/svg" 
        xmlns="http://www.w3.org/1999/xlink" 
        width="16" height="16">
          <path d="M511.700683 639.423111 191.917496 319.596945 319.830771 319.596945 511.700683 511.715521 703.570595 319.596945 831.48387 319.596945Z" p-id="8626" fill={fillColor}></path>
  </svg>
);
const HomeIcon = props => <Icon component={homeSvg} {...props} style={{marginRight: '0px'}}/>;
const ArrowIcon = props => <Icon component={arrowSvg} {...props}  />;
// const { TabPane } = Tabs;

// const baseURI = isProd
//   ? window.location.origin
//   : `http://${window.location.hostname}:3020`;
const optionsWith = [
  { label: '按站点', value: 'site' },
  { label: '按城市', value: 'city' },
];


const HOT = ['HOT', '县', '市', 'New'];
const StationsItem = (item, action) => {
  const changeGroup = (event, data) => {
    action.changeGroup && action.changeGroup(event, data);
  };
  return (
    <li
      key={item.groupID}
      style={{ marginRight: `${item.style.marginRight}px` }}
    >
      <a
        className={classNames(styles.city_select, styles.showTip)}
        title={item.city}
        index={item.promoteIndex}
        style={{ fontSize: '13px' }}
        onClick={event => changeGroup(event, item)}
      >
        {item.groupName}
        {item.promoteTip && HOT.includes(item.promoteTip) && (
          <div title={`${item.promoteTip}`}>{item.promoteTip}</div>
        )}
      </a>
    </li>
  );
};

const Stations = props => {
  const data = props.data.stations;
  const [targetOffset, setTargetOffset] = useState(undefined);
  const cityPane = useRef(null);
  const cityContent = useRef(null);
  const [defaultTab, setDefaultTab] = useState('site');
  useEffect(() => {
    setTargetOffset(cityPane.current.clientHeight / 2);
  }, []);

  const handleClick = (event, link) => {
    event.preventDefault();
  };

  const handleTabChange = event => {
    setDefaultTab(event.target.value);
  };
  return (
    <>
      <div className={styles.focusStations}>
        <ul>
          {Array.isArray(data)
            ? data.map(item => StationsItem(item, props.action))
            : null}
        </ul>
      </div>
      <Radio.Group
        options={optionsWith}
        optionType="button"
        buttonStyle="solid"
        size="small"
        value={defaultTab}
        onChange={handleTabChange}
        style={{ marginTop: '6px' }}
      />
      <div style={{ maxWidth: '520px', position: 'relative' }}>
        {defaultTab === 'site' ? (
          <div
            className={classNames(styles.city_pane, styles.station_container)}
            ref={cityPane}
          >
            <Anchor
              affix={false}
              onClick={handleClick}
              targetOffset={targetOffset}
              getContainer={() => cityContent.current}
            >
              <ul className={styles.py}>
                {props.data &&
                  props.data.siteCityList &&
                  props.data.siteCityList.letters}
              </ul>
              <div
                className={styles.cityContent}
                style={{ height: '335px' }}
                ref={cityContent}
              >
                {props.data &&
                  props.data.siteCityList &&
                  props.data.siteCityList.content}
              </div>
            </Anchor>
          </div>
        ) : (
          <div className="city_pane city_container">
            {props.data.citySelector}
          </div>
        )}
        <Spin spinning={props.loading} tip="加载中" />
      </div>
    </>
  );
};

const renderTitle = ({
  title,
  action,
  data,
  loading,
  setLoading,
  collapsed,
  fixedHeader,
  config,
}) => {
  // eslint-disable-next-line react-hooks/rules-of-hooks,no-debugger
  // eslint-disable-next-line react-hooks/rules-of-hooks
  const [visible, setVisible] = useState(false);
  const header = document.querySelector('.ant-pro-fixed-header');
  let marginLeft = '148px';
  const DEFAULT_MIXIN_WIDTH = 48;
  if (header && fixedHeader) {
    const documentWidth = document.body.clientWidth;
    if (documentWidth === header.offsetWidth && !collapsed) {
      marginLeft = '148px';
    } else {
      /* eslint-disable */
      const deltWidth = document.body.clientWidth - header.offsetWidth;
      marginLeft = collapsed
        ? '0px'
        : deltWidth === DEFAULT_MIXIN_WIDTH
        ? `100px`
        : `${deltWidth}px`;
    }
  }
  let loaded = !!(
    (data && !data.stations) ||
    (Array.isArray(data.weathers) && data.weathers.length === 0)
  );
  if (config && config.userInfo && config.userInfo.site === '') {
    loaded = false;
  }
  return (
    <>
      <span className={styles['header-title']} style={{ marginLeft }}>
        {title}
      </span>
      <Spin
        indicator={antIcon}
        spinning={loaded}
        size="small"
        style={{ marginLeft: '10px' }}
        tip="加载中"
        wrapperClassName={styles.spinLoadding}
      >
        <div className={styles.cityContent}>
          <Popover
            placement="bottomLeft"
            trigger="click"
            content={
              <Stations
                data={data}
                action={action}
                loading={loading}
                setLoading={setLoading}
              />
            }
            arrowPointAtCenter
            overlayClassName={styles.stationsWrapper}
            onVisibleChange={visible => setVisible(visible)}
          >
            {Array.isArray(data.stations) && data.stations.length > 0 ? (
              <span className={styles.siteName}>
                <img src="https://panda-water.cn/web4/assets/images/depart.svg" />
                <span>{data.currentStationName}</span>
                <img
                  src="https://panda-water.cn/web4/assets/images/下.svg"
                  style={{
                    transform: !visible ? `rotate(0deg)` : `rotate(180deg)`,
                  }}
                />
              </span>
            ) : null}
          </Popover>
          {data.weathers && Object.keys(data.weathers).length > 0 ? (
            <span
              className={styles.weatcher}
              style={{
                borderLeft:
                  data.stations.length === 0 ? '0px' : '1px solid #aaaaaa',
              }}
            >
              <img src={data.weathers && data.weathers.icon} />
              <span>{data.weathers && data.weathers.text}</span>
            </span>
          ) : null}
        </div>
      </Spin>
    </>
  );
};

const renderSite = ({data, config, loading, setLoading, action}) => {
  const [visible, setVisible] = useState(false);
  let loaded = !!((data && !data.stations) || (Array.isArray(data.weathers) && data.weathers.length === 0));
  if (config && config.userInfo && config.userInfo.site === '') {
    loaded = true;
  };
  return (
    <>
      <Spin 
            indicator={antIcon}
            spinning={loaded}
            size="small"
            style={{ marginLeft: '10px' }}
            tip=""
            wrapperClassName={styles.spinLoadding}>
              <>
              <Popover
                placement="bottomLeft"
                trigger="click"
                content={
                  <Stations
                    data={data}
                    loading={loading}
                    setLoading={setLoading}
                    action={action}
                  />
                }
                arrowPointAtCenter
                overlayClassName={styles.stationsWrapper}
                onVisibleChange={visible => setVisible(visible)}
              >
              {
                 Array.isArray(data.stations) && data.stations.length > 0 ? (
                  <div className={layoutStyles.toggleSite}>
                    <img src={require('../assets/basic/site.png')} className={layoutStyles.site}/>
                    <span className={layoutStyles.name}>{data.currentStationName}</span>
                    <ArrowIcon className={layoutStyles.arrow} fillColor="#fff"
                        style={{
                          transform: !visible ? `rotate(0deg)` : `rotate(180deg)`,
                        }}
                    />
                  </div>
                 ): null
              }
              </Popover>
              </>
          </Spin>
          {data.weathers && Object.keys(data.weathers).length > 0 ? (
                <span
                  className={layoutStyles.weatcher}
                  style={{
                    borderLeft:
                      data.stations.length === 0 ? '0px' : '1px solid rgba(256, 256, 256, 0.3)',
                  }}
                >
                <img src={data.weathers && data.weathers.icon} className={layoutStyles.icon}/>
                <span className={layoutStyles.text}>{data.weathers && data.weathers.text}</span>
              </span>
            ) : null}
    </>
    
  )
}

const BasicLayout = props => {
  /* eslint-disable no-unused-vars */

  // const [currentRoutes, setCurrentRoutes] = useState([]);
  const [collapse, setCollapse] = useState(false);
  const [toggleSystem, setToggleSystem] = useState(false);
  const [cityData, setCityData] = useState({});
  const [siteLoading, setSiteLoading] = useState(false);
  const [selectIndex, setSelectIndex] = useState(-1);
  const [childrenRoutes, setChildrenRoutes] = useState([])
  const [selectedKeys, setSelectedKeys] = useState([])
  const [siteAction, setSiteAction] = useState(
    () => new Site(props, setSiteLoading),
  );
  const history = useHistory();
  const currentRoutes = props.route.routes[props.currentMenuIndex];

  useEffect(() => {
    const initSelectRoute = findPathByLeafId(
      `${props.location.pathname}`,
      [currentRoutes],
      '',
      'path',
      true,
    );

    const parentMenuName = initSelectRoute && initSelectRoute.parent.label;
    if(parentMenuName) {
      let currentChildrenRoute = currentRoutes.routes.find(item => item.name === parentMenuName);
      if(!currentChildrenRoute) {
        currentChildrenRoute = initSelectRoute
      }
      let childrenName = currentChildrenRoute ? currentChildrenRoute.name: parentMenuName;
      const initSelectIndex = currentRoutes.routes.findIndex(item => item.name === childrenName);
      setSelectIndex(initSelectIndex);
      currentChildrenRoute && currentChildrenRoute.routes ? setChildrenRoutes(currentChildrenRoute.routes): setChildrenRoutes([currentChildrenRoute])
    } else {
      setChildrenRoutes([
        {
          icon: <HomeIcon/>,
          path: props.location.pathname,
          name: '首页'
        }
      ])
    }
  }, []);

  useEffect(() => {
    siteAction.setGlobalConfig(props.global);
    if (!Cookies.get('token')) {
      history.replace(`/user/login?client=${props.global.get('client')}`);
      props.logout();
      return;
    }
    if (
      props.global &&
      props.global.userInfo &&
      props.global.userInfo.token !== null &&
      props.global.userInfo.loginName &&
      Object.keys(cityData).length === 0
    ) {
      siteAction.getCityStationsForUser().then(res => {
        setCityData(res);
      });
    }
  }, [props.global]);
  // setChildrenRoutes(currentChildrenRoute)

  window.share.event.on('goHome', url => {
    setChildrenRoutes([
      {
        icon: <HomeIcon/>,
        path: url,
        name: '首页'
      }
    ])
  });

  useEffect(() => {
    window.share.event.on('updateSite', res => setCityData(res));
    return () => {
      window.share.event.removeAllListeners('updateSite');
      // sessionStorage.removeItem('tabNav')
      // sessionStorage.removeItem('tabActiveKey')
    };
  }, []);

  const handlerSecond = (item, index) => {
    
    let current = void 0;
    if(item && item.routes) {
      setChildrenRoutes(item.routes);
      current = item.routes[0];
    } else{
      setChildrenRoutes([item]);
      current = item;
    }
    window.history.pushState(null, '', `/civbase${current.path}`);
    setSelectIndex(index);
  }

  const handleToggleSystem = () => {
    setToggleSystem(!toggleSystem);
  }

  const handleUpdateCurrentIndex = index => {
    props.updateCurrentIndex(index);
    window.share && window.share.event.emit('trigger:updateMenuIndex', index);
    store.set('updateMenuIndex', index);
  }
  
  const handleSelectMenuItem = (item) => {
    if(item.indexOf('web_console') === -1) {
      window.history.pushState(null, '', `/civbase${item}`);
    }
  }

  const handlerSelectMenu = ({ item, key, keyPath, domEvent }) => {
    setSelectedKeys([key]);
    window.history.pushState(null, '', `/civbase${key}`);
  }
  
  const renderChildrenMenu = (route) => {
    const routes =  (route.routes || []).map(item => {
      item.key = item.href;
      return item;
    });
    const menu = (
      <Menu onClick={handlerSelectMenu} selectedKeys={selectedKeys}>
        {
          (routes || []).map((item, index) => {
            return (
              <>
                 <Menu.Item key={item.href}>
                  <div>
                    <span></span>
                    <span>{item.name}</span>
                  </div>
                </Menu.Item>
                <Divider style={{width: '80%', minWidth: '80%', margin: '0 auto'}}></Divider>
              </>
            )
          })
        }
      </Menu>
    )
    return (
      <Dropdown overlay={menu} overlayClassName="ant-dropdown-custom-menu">
        <div>
          {
            /.svg/.test(route.extData.icon) ?<ReactSVG src={route.extData.icon} style={{width: '18px', height: '18px', position: 'relative', top: '-3px'}}/>: <img src={route.extData.icon} style={{width: '18px', height: '18px'}}/>
          }
          <span style={{marginLeft: '8px'}}>{route.name} </span>
          <DownOutlined style={{position: 'relative', right: '-10px'}}/>
        </div>
      </Dropdown>
    )
  }

  const  handlerIndustry = event => {
    props.global.get('userInfo.site') ? history.push(`/industry`) : void 0;
  };

  const logo = props.global.get('bannerLogo')? window.globalConfig.transformDevAssetsBaseURL(props.global.get('bannerLogo')): defaultSetting.logo;
  

  return (
    <SecurityLayout loading>
      {collapse && props.global.loginTemplate === '新春 - 智联.html' ? null : (
        props.global.loginTemplate === '新春 - 智联.html' && (
        <img
          src="https://panda-water.com/web4/assets/images/%E6%96%B0%E5%B9%B4%E4%B8%BB%E9%A2%98/%E6%96%B0%E5%B9%B4%E5%BF%AB%E4%B9%90.gif"
          style={{
            width: '120px',
            position: 'absolute',
            bottom: '40px',
            zIndex: 500,
            left: '12px',
          }}
        />)
      )}
      <div className={layoutStyles.basicLayout}>
        <div className={layoutStyles['layout-has-slider']} style={{minHeight: '100%'}}>
          <div style={{
            width: '100px', 
            overflow: 'hidden', 
            flex: '0 0 100px', 
            maxWidth: '100px', 
            minWidth: '100px',
            transition: 'background-color 0.3s ease 0s, min-width 0.3s ease 0s, max-width 0.3s cubic-bezier(0.645, 0.045, 0.355, 1) 0s'
          }}></div>
          <aside className={`${layoutStyles.layoutSlider} ${layoutStyles.fixed}`}>
            <div className={layoutStyles['layout-slider-childern']}>
              <div className={layoutStyles.sliderMenu}>
                <ul className="menu" style={{minHeight: '0px', marginBottom: '0px'}}>
                  <li className={classNames('menu-item')} onClick={handleToggleSystem}>
                      <a>
                        <img src={currentRoutes && currentRoutes.extData && currentRoutes.extData.icon}/>
                        <span>{currentRoutes && currentRoutes.name}</span>
                      </a>
                    </li>
                </ul>
                <div className={layoutStyles.splitLine}></div>
                <ul className="menu menu-vertical">
                  {
                    currentRoutes && (currentRoutes.routes || []).map((item, index) => {
                      return (
                        <>
                          <li className={classNames("menu-item", { [layoutStyles['active']]: index === selectIndex })} onClick={() => handlerSecond(item, index)}>
                            <a>
                              <img src={item.extData.icon}/>
                              <span>{item.name}</span>
                            </a>
                          </li>
                        <Divider style={{border: '1px solid rgb(49, 53, 62, 0.3)', margin: '0 auto', minWidth: '80%', width: '80%'}}/>
                        </>
                      )
                    })
                  }
                </ul>
              </div>
            </div>    
          </aside>
          <div className={layoutStyles.layout}>
          <Panel
            visible={toggleSystem}
            keyboard
            onClose={ () => setToggleSystem(false)}
            currentMenuIndex={props.currentMenuIndex}
            onChange={index => handleUpdateCurrentIndex(index)}
            data={props.route.routes || []}
          />
            <div className={layoutStyles['basicLayout-content']}>
                <div className={layoutStyles['ant-page-container']} style={{paddingTop: '46px'}}>
                  <div className={layoutStyles['ant-pro-page-container-warp']}>
                    <div className={layoutStyles['ant-page-header']} style={{position: 'fixed'}}>
                        <img src={require('../assets/basic/图层 998@2x.png')} style={{width: '100%'}}/>
                        <div className={layoutStyles.header}>
                          <a class={layoutStyles.logo} onClick={handlerIndustry}>
                            <img src="https://panda-water.com/web4/assets/images/logo/单独图案-白色.svg" alt="logo"/>
                          </a>
                          <div className={layoutStyles.title}>{props.global.title}</div>
                          {
                            renderSite({
                              data: cityData,
                              config: props.global,
                              loading: siteLoading,
                              setLoading: setSiteLoading,
                              action: siteAction,
                            })
                          }
                          <RightContent/>
                        </div>
                       
                    </div>
                  </div>
                  <div className={layoutStyles['menu-item-children']}>
                     <Tabs defaultActiveKey="1" tabBarGutter={30} onTabClick={(event) => handleSelectMenuItem(event)}>
                     {childrenRoutes.map((item, index) => (
                          <>
                            <TabPane 
                              className="only-last-children"
                              tab={
                                <>
                                  {
                                     item.routes ? renderChildrenMenu(item) : (
                                      <>
                                        {
                                          item.icon ? item.icon: /.svg/.test(item.extData.icon) ? <ReactSVG src={item.extData.icon} style={{width: '18px', height: '18px'}}/>: <img src={item.extData.icon} style={{width: '18px', height: '18px'}}/>
                                        }
                                        <span className={layoutStyles['menu-item-name']}>{item.name}</span>
                                      </>
                                    )
                                  }
                                </>
                              }
                              key={item.path}
                              disabled={index === 28}>
                           </TabPane>
                          </>
                      ))}
                     </Tabs>
                    
                  </div>
                  <div className="ant-pro-grid-content">
                        <div className="ant-pro-grid-content-children">
                              <div className="ant-pro-page-container-children-content">
                                  <div id="micro-container" className="subapp-container" style={{height: '91vh', marginTop: '70px', position: 'relative'}}/>
                              </div>
                        </div>
                    </div>
                </div>  
            </div>
          </div>
        </div>
      </div>
    </SecurityLayout>
  );
};
const mapStateToProps = state => ({
  global: state.getIn(['global', 'globalConfig']),
  settings: state.getIn(['global', 'defaultSetting']),
  collapsed: state.getIn(['global', 'collapsed']),
  menu: state.getIn(['global', 'menu']),
  currentMenuIndex: state.getIn(['global', 'currentMenuIndex']),
  flatMenu: state.getIn(['global', 'flatMenu']),
  authValidate: state.getIn(['global', 'authValidate']),
  pathname: state.getIn(['global', 'pathname']),
  selectedKeys: state.getIn(['global', 'selectedKeys']),
  openKeys: state.getIn(['global', 'openKeys']),
  complexConfig: state.getIn(['global', 'complexConfig']),
  complexPathName: state.getIn(['global', 'complexPathName']),
  microMounted: state.getIn(['global', 'microMounted']),
});
const mapDispatchToProps = dispatch => ({
  updageSetting(setting) {
    dispatch(actionCreators.updageSetting(setting));
  },
  updateConfig(config) {
    dispatch(actionCreators.getConfig(config));
  },
  updateCurrentIndex(index) {
    dispatch(actionCreators.updateCurrentIndex(index));
  },
  updateCollapsed(collapsed) {
    dispatch(actionCreators.updateCollapsed(collapsed));
  },
  updateAuthValidate(auth) {
    dispatch(actionCreators.updateAuthValidate(auth));
  },
  updatePathname(pathname) {
    dispatch(actionCreators.updatePathname(pathname));
  },
  updateSelectedKeys(keys) {
    dispatch(actionCreators.updateSelectedKeys(keys));
  },
  updateOpenKeys(keys) {
    dispatch(actionCreators.updateOpenKeys(keys));
  },
  updateComplexConfig(config) {
    dispatch(actionCreators.updateComplexConfig(config));
  },
  updateComplexPathName(pathname) {
    dispatch(actionCreators.updateComplexPathName(pathname));
  },
  logout() {
    dispatch(actionCreators.logout());
  }
});
export default connect(
  mapStateToProps,
  mapDispatchToProps,
)(BasicLayout);
